/** This class makes a Cartesian Axis on which to draw signals.  It was inspired 
 * and adapted from a Sun Java applet example called 'Arc Test'
 * @author Arthur Asuncion
 */

import java.awt.*;


public class CartesianCanvas extends Canvas {
	
	// size of box, and offsets of the x and y axes
	static final int BOX_SIZE = 20;
	static final int X_AXIS_OFFSET = 120;
	static final int Y_AXIS_OFFSET = 140;
	
	boolean drawGrid = true;
	boolean drawSignal = false;

	
	float[] oldXCoord;
	float[] oldYCoord;
	int[] xCoordinates;
	int[] yCoordinates;
	short nSamples;
	long nIterations;
	
	float yScale;
	String title;
	
	/** Constructor for the canvas
	 * 
	 * @param scale How big the signal should appear on the screen
	 * @param t The title of the canvas
	 */
	public CartesianCanvas(float scale, String t)
	{
		yScale = scale;
		title = t;
		
	}

	
	/** Paints the signal
	 *  @param Graphic the graphic on which to paint the signal
	 */
	public void paint(Graphics g){
		
		Rectangle r = getBounds();

		this.setBackground(Color.white);
		
		// draws the grid
		if (drawGrid){
			
			int xLines = r.height / BOX_SIZE;
			int yLines = r.width / BOX_SIZE;
			
			g.setColor(Color.lightGray);
			
			for (int i = 0; i <= xLines; i++) {
				g.drawLine(0, i * BOX_SIZE, r.width, i * BOX_SIZE);
			}
			
			for (int i = 0; i <= yLines; i++) {
				g.drawLine(i * BOX_SIZE, 0, i * BOX_SIZE, r.height);
			}
			
		}
		
		// draws the signal
		if (drawSignal) {
			g.setColor(Color.blue);
			g.drawPolyline(xCoordinates, yCoordinates, nSamples);
			
			
			for (int i = 0; i < nSamples; i = i + 2) {
				g.setColor(Color.blue);
				g.fillOval(xCoordinates[i] - 1, yCoordinates[i] - 1, 2, 2);
			}
		}
		

		g.setColor(Color.blue);
		g.drawString(title + ": ", X_AXIS_OFFSET - 5 * BOX_SIZE, BOX_SIZE);

	}
	
	
	/** Redraws the canvas with the specified parameters
	 * @param xCoord x-coordinates of the samples
	 * @param yCoord y-coordinates of the samples
	 * @param size number of samples
	 */
	public void redraw(float[] xCoord, float[] yCoord, short size) 
	{
		drawSignal = true;
		nSamples = size;
		oldXCoord = xCoord;
		oldYCoord = yCoord;
		
		// have to transform the coordinates
		Doubleton transformedCoordinates = 
				transformCoordinates(xCoord, yCoord, size);
		xCoordinates = (int[])transformedCoordinates.getFirst();
		yCoordinates = (int[])transformedCoordinates.getSecond();
		
		repaint();
	}
	
	/** Draws/Undraws the grid
	 * @param boolean grid true if grid is to be drawn
	 */
	public void drawGrid(boolean grid) {	
		drawGrid = grid;
		repaint();
	}
	

	/** Clears the canvas of the previous display
	 */
	public void clearCanvas() {	
		drawSignal = false;
		nIterations = 0;
		repaint();
	}
	
	/** Transforms the coordinates to make it look right on the GUI
	 * @param float[] xCoord x-coordinates to transform
	 * @param float[] yCoord y-coordinates to transform
	 * @param short numberNodes number of points
	 * @return Doubleton the transformed x and y coordinates
	 */
	Doubleton transformCoordinates(float[] xCoord, float[] yCoord, 
									short numberNodes){
		
		int[] transXCoord = new int[numberNodes];
		int[] transYCoord = new int[numberNodes];
		
		// Logic here
		for (int i = 0; i < numberNodes; i++) {
			transXCoord[i] = (int)Math.floor(BOX_SIZE * xCoord[i] / 300.0) 
							+ X_AXIS_OFFSET;
			transYCoord[i] = -1 * (int)Math.floor(BOX_SIZE * yCoord[i] * yScale / (double) Short.MAX_VALUE) 
							+ Y_AXIS_OFFSET / 2;
		}
		
		return new Doubleton(transXCoord, transYCoord);
	}
	
}
