/**
 * @author Arthur Asuncion
 *
 * This class encapsulates all of the functionality associated with the 
 * Daubechies 4 wavelet transform.  The implementation was mainly based
 * on "Ripples in Mathematics", by A. Jensen and A.la Cour-Harbo
 */
public class Daubechies implements Wavelet {
	
	// These are constants that are used heavily by the Daubechies transform
	public static final float SQRT3 = (float) Math.sqrt(3);
	public static final float SQRT3OVER4 = (float) (Math.sqrt(3) / 4);
	public static final float SQRT3MINUS2OVER4 = (float) ((Math.sqrt(3) - 2)/4);
	public static final float SQRT3MINUS1OVERSQRT2 = (float) ((Math.sqrt(3) - 1) / Math.sqrt(2));
	public static final float SQRT3PLUS1OVERSQRT2 = (float) ((Math.sqrt(3) + 1) / Math.sqrt(2));
	
	/** This method decomposes a signal using multiple applications of the Daubechies 4 transform
	 * 
	 * @param S the signal to decompose, as a float array.  The coefficients will be written on this array
	 * @param size the number of samples in the array.
	 */
	public void decompose(float[] S, int size)
	{
		// The transform is applied repeatedly to the "average" section of the array
		// until the "average" section only has 4 samples left
		for (int i = size; i >= 4; i = i / 2)
		{
			daubechiesTransform(S, i);
		}
		
	}
	
	/** This implementation of the Daubechies 4 transform is based on "Ripples in Mathematics"
	 * 
	 * @param S the samples to transform, as a float array.  The coefficients will be written on this array
	 * @param size the number of samples in the array
	 */
	static void daubechiesTransform(float[] S, int size)
	{
		float[] s = new float[size/2]; // This is the "average" array
		float[] d = new float[size/2]; // This is the "change" array
		
		for (int n = 0; n < size/2; n++) {
			s[n] = S[2*n] + SQRT3 * S[2*n + 1];
		}
		
		// This statement is done separately from the for-loop since there is a
		// periodic extension: s[size/2 - 1]
		d[0] = S[1] - SQRT3OVER4 * s[0] - SQRT3MINUS2OVER4 * s[size/2 - 1];
		
		for (int n = 1; n < size/2; n++) {
			d[n] = S[2*n + 1] - SQRT3OVER4 * s[n] - SQRT3MINUS2OVER4 * s[n-1];
		}
			
		for (int n = 0; n < size/2 - 1; n++){
			s[n] = s[n] - d[n+1];
		}
		
		s[size/2 - 1] = s[size/2 - 1] - d[0];
		
		//Normalization and writing the results back to the S array
		for (int n = 0; n < size/2; n++) {
			S[n] = SQRT3MINUS1OVERSQRT2 * s[n];
			S[n + size/2] = SQRT3PLUS1OVERSQRT2 * d[n];
		}

	}
	
	/** This inverse Daubechies 4 transform simply undoes the Daubechies 4 transform
	 * 
	 * @param S the samples to reconstruct, as a float array.  The signal will be written on this array
	 * @param size the number of samples in the array
	 */
	public void reconstruct(float[] S, int size)
	{
		// I simply reversed the decomposeWithDaubechies function!
		for (int i = 4; i <= size; i = i * 2)
		{
			inverseDaubechiesTransform(S, i);
		}
	}
	
	/**
	 * 
	 * @param S the samples to reconstruct, as a float array.  The reconstructed samples will be written on this array
	 * @param size the number of samples in the array
	 */
	static void inverseDaubechiesTransform(float[] S, int size)
	{
		// I simply reversed every part of the daubechiesTransform function, and it works!
		
		float[] s = new float[size/2];
		float[] d = new float[size/2];
		
		// Denormalizing
		for (int n = 0; n < size/2; n++) {
			d[n] = (float) (S[n + size/2] / (SQRT3PLUS1OVERSQRT2));
			s[n] = (float) (S[n] / (SQRT3MINUS1OVERSQRT2));
		}
		
		s[size/2 - 1] = s[size/2 - 1] + d[0];
		
		for (int n = 0; n < size/2 - 1; n++)
		{
			s[n] = s[n] + d[n+1];
		}
		
		// Here, we start to reconstruct the signal
		for (int n = 1; n < size/2; n++) 
		{
			S[2*n + 1] = (float) (d[n] + SQRT3MINUS2OVER4 * s[n-1] + SQRT3OVER4 * s[n]) ;
		}
		
		S[1] = (float) (d[0] + SQRT3OVER4 * s[0] + SQRT3MINUS2OVER4 * s[size/2 - 1]);
		
		for (int n = 0; n < size/2; n++) {
			S[2*n] = (float) (s[n] - SQRT3 * S[2*n + 1]);
		}

	}
	
	
	// This method simply tests whether or not the Daubechies transforms work
	public static void main(String[] args) {
		
		Daubechies d = new Daubechies();
		
		int size = 32;
		
		float[] s = {3,1,0,4,8,6,9,9, -2, 23, 5, 2, 6, 1, 5, 4,
					1, 4, -28, 4, 3, 5, 2, -9, 3, -6, 3, 4, 2, 3, 4, 1};
		
		System.out.println("\nOriginal Signal:");
		
		for (int i = 0; i < size; i++)
		{
			System.out.print(s[i] + " ");
		}
		
		
		d.decompose(s, size);
		
		
		System.out.println("\nDaubechies 4 Transform applied on signal (ordered coefficients):");
		
		for (int i = 0; i < size; i++)
		{
			System.out.print(s[i] + " ");
		}
		
		
		d.reconstruct(s, size);
		
		System.out.println("\nInverse Daubechies 4 Transform is performed, yielding original signal:");
		
		for (int i = 0; i < size; i++)
		{
			System.out.print(s[i] + " ");
		}
	}
}
