/*
 * Created on Feb 17, 2005
 *
 */
/**
 * @author Arthur Asuncion
 *
 * This class encapuslates all of the functionality of the In-Place Haar wavelet transform.
 */

public class Haar implements Wavelet{
	
	/** This method decomposes a signal completely into wavelet coefficients 
	 *  using multiple Haar wavelet transforms. An in-place algorithm is used.
	 *  The coefficients that are created (and put in the input array) are 
	 *  unordered, in a butterfly pattern.
	 *  <p>The implementation of this in-place algorithm was inspired by "Wavelets Made Easy", 
	 *  by Yves Nievergelt, and the faster 'lifted' version was inspired by:
	 *  http://www.control.auc.dk/~alc/html/2__haar_transform_through_lift.html
	 * 
	 * 
	 * @param s The float array that holds the samples.  This method will write the calculated coefficients in this array.
	 * @param size The size of the float array
	 */
	public void decompose(float[] s, int size)
	{
		// This outer loop controls the step size used to perform the Haar transforms
		// Remember that this decomposition produces coefficients in a butterfly pattern
		for (int i = 2; i <= size; i = i * 2) 
		{
			// This inner loop performs one iteration of the Haar transform each time
			for (int j = 0; j <= size - i; j = j + i) 
			{
				s[j] = (float) (((double)((s[j] + s[j + i/2]))) / 2.);
				s[j + i/2] =  s[j] - s[j + i/2];
			}	
		}
		
		orderCoefficients(s, size);
	}
	
	
	/** This method perfectly reconstructs the signal from the Haar wavelet coefficents.
	 *  This reconstruction is also in-place.  The coefficents have to be in butterfly-ordering
	 *  for this method to work correctly.
	 * 
	 * @param s		The float array that holds the Haar wavelet coefficients.  This method will write the signal back into this array.
	 * @param size	Size of the float array.
	 */
	public void reconstruct(float[] s, int size)
	{
		unorderCoefficients(s, size);
		
		// I just reversed every part of the decomposeWithHaar method and it works!
		for (int i = size; i >= 2; i = i / 2)
		{
			for (int j = size - i; j >= 0; j = j - i)
			{		
				s[j + i/2] = s[j] - s[j + i/2];
				s[j] = 2 * s[j] - s[j + i/2];	
			}
		}
	}
	
	
	/** This method takes an array with coefficients in butterfly pattern and reorders the coefficients
	 *  by frequency (from lowest to highest).
	 * 
	 * @param s		The float array that holds the coefficients.  This method will write the ordered coefficients over this array.
	 * @param size	The size of the float array.
	 */
	static void orderCoefficients(float[] s, int size)
	{
		float[] temp = new float[size];
		int tempCount = 0;
		
		temp[tempCount++] = s[0];
		
		// These loops are pretty complicated.  It took me a couple of hours to figure out.
		// The outer loop controls the step size 
		// (starting from size/2 and going down to 1 by halving the step size at each iteration).
		for (int i = size/2; i >= 1; i = i / 2)
		{
			// Each step size represents a frequency.  So the frequencies are reordered
			// by stepping over the array by the specified step sizes (the bigger the step size,
			// the lower the frequency).  
			for (int j = i; j <= size - i; j = j + 2*i)
			{
				temp[tempCount++] = s[j];
			}
		}
		
		// Writes the ordered coefficients back into the output array
		for (int i = 0; i < size; i++)
		{
			s[i] = temp[i];
		}
	}
	
	/** This method takes ordered coefficients and 'unorders' them back to the butterfly pattern.
	 *  This is needed because the function, "reconstructWithHaar", needs a butterfly ordered array
	 *  to function.  
	 * @param s		The float array that contains the ordered coefficients.  This method will write the coefficients back into this array using the butterfly pattern.
	 * @param size	The size of the float array.
	 */
	static void unorderCoefficients(float[] s, int size)
	{
		// This implementation is simply the reverse of the 'orderCoefficients' implementation
		
		float[] temp = new float[size];
		
		temp[0] = s[0];
		int tempCount = size - 1;
		
		for (int i = 1; i <= size/2; i = i * 2)
		{
			for (int j = size - i; j >= i; j = j - 2*i)
			{
				temp[j] = s[tempCount--];
			}
		}
		
		for (int i = 0; i < size; i++)
		{
			s[i] = temp[i];
		}
	}
	
	
	
	// This main method simply tests some of the functions in this class
	public static void main(String[] args) {
		
		Haar h = new Haar();
		int size = 32;
		
		float[] s = {3,1,0,4,8,6,9,9, -2, 23, 5, 2, 6, 1, 5, 4,
					1, 4, -28, 4, 3, 5, 2, -9, 3, -6, 3, 4, 2, 3, 4, 1};
		
		
		System.out.println("\nOriginal Signal:");
		
		for (int i = 0; i < size; i++)
		{
			System.out.print(s[i] + " ");
		}
		
		
		h.decompose(s, size);
		
		
		System.out.println("\nIn-place Haar Transform applied on signal (unordered coefficients):");
		
		for (int i = 0; i < size; i++)
		{
			System.out.print(s[i] + " ");
		}
		
		
	
		orderCoefficients(s, size);
		
		
		System.out.println("\nCoefficients are ordered from low frequencies to high frequencies:");
		
		for (int i = 0; i < size; i++)
		{
			System.out.print(s[i] + " ");
		}
		
		
		unorderCoefficients(s, size);
		
		System.out.println("\nCoefficients are returned to the butterfly pattern:");
		
		for (int i = 0; i < size; i++)
		{
			System.out.print(s[i] + " ");
		}
		
		
		h.reconstruct(s, size);
		
		System.out.println("\nInverse transform is performed, yielding original signal:");
		
		for (int i = 0; i < size; i++)
		{
			System.out.print(s[i] + " ");
		}
		
	}
	
}
