/**This class makes the view of the GUI
 * @author Arthur Asuncion
 */

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

public class SignalView {
	
	// fields for the GUI
	private JApplet theApplet;
	
	private JPanel controlPanel;
	private JPanel optionsPanel;
	private JPanel crossSynthPanel;
	private JPanel filterPanel;
	private JPanel subControlPanel;
	private JPanel bottomPanel;
	private JPanel canvasPane;
	private JPanel msgPanel;
	private JPanel basisPanel;
	private JPanel effectsPanel;
	
	private JButton loadFile;
	private JButton start;
	private JButton pause;
	private JButton stop;
	
	private ButtonGroup sourceOption;
	private JRadioButton sourceMic;
	private JRadioButton sourceFile;

	
	private ButtonGroup waveletOption;
	private JRadioButton haar;
	private JRadioButton daubechies;
	
	private JButton loadCrossSynthesisFile;
	
	private JCheckBox crossSynthesize;
	private JCheckBox simulateCompression;
	private JCheckBox denoise;
	private JCheckBox modulate;
	private JCheckBox threshold;
	private JCheckBox removeClicks;
	private JCheckBox filter;
	private JCheckBox hideView;
	
	private JSpinner[] filterSpinners;

	
	private JLabel msg;
	
	private TextField numberNodes;
	private TextField numberIterations;
	private TextField numberRestarts;

	private CartesianCanvas canvas;
	private CartesianCanvas canvas2;
	private WaveletCanvas canvas3;
	
	private boolean isPaused;
	private boolean isStopped;
	
	
	/** Constructor
	 * @param JApplet app the controller applet 
	 */
	public SignalView(JApplet app) {
		
		theApplet=app;
		
		// Canvases to show files
		canvas = new CartesianCanvas(3, "Signal");
		canvas2 = new CartesianCanvas((float)0.8, "Coefficients");
		canvas3 = new WaveletCanvas();
		
		isPaused = false;
		isStopped = false;

		app.getContentPane().setLayout(new GridLayout(4, 1));
		
		// All the panels
		controlPanel = new JPanel(new FlowLayout());
		bottomPanel = new JPanel(new GridLayout(4,1));
		subControlPanel = new JPanel();
		msgPanel = new JPanel();
		crossSynthPanel = new JPanel();
		optionsPanel = new JPanel(new FlowLayout());
		filterPanel = new JPanel();
		basisPanel = new JPanel();
		effectsPanel = new JPanel();
		
		
		//	This section sets up the control panel
		JLabel sourceLabel = new JLabel("Source: ");
		
		sourceOption = new ButtonGroup();
		sourceMic = new JRadioButton("Mic");
		sourceFile = new JRadioButton("File");
		
		sourceFile.setSelected(true);
		
		sourceOption.add(sourceMic);
		sourceOption.add(sourceFile);
		
		loadFile = new JButton("Load WAV");
		start = new JButton("Start");
		pause = new JButton("Pause");
		stop = new JButton("Stop");

		subControlPanel.add(sourceLabel);
		subControlPanel.add(sourceMic);
		subControlPanel.add(sourceFile);
		subControlPanel.add(loadFile);
		subControlPanel.add(start);
		subControlPanel.add(pause);
		subControlPanel.add(stop);
		

		
		
		waveletOption = new ButtonGroup();
		haar = new JRadioButton("Haar");
		daubechies = new JRadioButton("Daubechies");
		waveletOption.add(haar);
		waveletOption.add(daubechies);
		
		daubechies.setSelected(true);
		
	

		crossSynthesize = new JCheckBox("Cross-Synthesize with Mic");		

		
		// Options (Effects) Panel
		JLabel waveletLabel = new JLabel("Wavelet: ");
		JLabel effectsLabel = new JLabel("Effects: ");
		simulateCompression = new JCheckBox("Simulate Compression");
		denoise = new JCheckBox("Denoise");
		modulate = new JCheckBox("Modulate");
		threshold = new JCheckBox("Threshold");
		removeClicks = new JCheckBox("Remove Clicks");
		
		basisPanel.add(waveletLabel);
		basisPanel.add(haar);
		basisPanel.add(daubechies);
		
		effectsPanel.add(effectsLabel);
		effectsPanel.add(simulateCompression);
		effectsPanel.add(denoise);
		effectsPanel.add(threshold);
		effectsPanel.add(modulate);
		effectsPanel.add(crossSynthesize);
		effectsPanel.add(removeClicks);
		
		
		filter = new JCheckBox("Filter: Low >> High");
		
		filterSpinners = new JSpinner[10];
		filterPanel.add(filter);
		
		for (int i = 0; i < filterSpinners.length; i++)
		{
			filterSpinners[i] = new JSpinner(new SpinnerNumberModel(100, 0, 200, 1));
			filterPanel.add(filterSpinners[i]);
		}
		
		hideView = new JCheckBox(" Stop View");
		
		filterPanel.add(hideView);
		
	
		// This section sets up the message panel
		msg = new JLabel("Welcome!");
		msg.setForeground(Color.blue);
		
		msgPanel.add("center", msg);
		msgPanel.setBorder(BorderFactory.createLineBorder(Color.black));


		controlPanel.add(subControlPanel);
		controlPanel.add(basisPanel);

		optionsPanel.add(effectsPanel);
		
		// This section synthesizes all of the panels
		bottomPanel.add(controlPanel);
		bottomPanel.add(optionsPanel);
		bottomPanel.add(filterPanel);
		bottomPanel.add(msgPanel);

		
		app.getContentPane().add(canvas);
		app.getContentPane().add(canvas2);
		app.getContentPane().add(canvas3);

		app.getContentPane().add(bottomPanel);
		
	}
	
	/** This method adds a listener to the load file button
	 * @param ActionListener a listens for mouse clicks
	 */
	void addLoadListener(ActionListener a) {
		loadFile.addActionListener(a);
	}
	
	/** This method adds a listener to the start button
	 * @param ActionListener a listens for mouse clicks
	 */
	void addStartListener(ActionListener a) {
		start.addActionListener(a);
	}
	
	/** This method adds a listener to the pause button
	 * @param ActionListener a listens for mouse clicks
	 */
	void addPauseListener(ActionListener a) {
		pause.addActionListener(a);
	}
	
	
	/** This method adds a listener to the stop button
	 * @param ActionListener a listens for mouse clicks
	 */
	void addStopListener(ActionListener a) {
		stop.addActionListener(a);
	}

	/** Sets the message in the message panel
	 * @param String message message to put on label
	 */
	void setMsg(String message) {
		msg.setText(message);
	}
	
	/** Sets the GUI to load state (all the user can do is load a file)
	 */
	public void setLoadState() {
		
		loadFile.setEnabled(true);

		start.setEnabled(false);
		pause.setEnabled(false);
		stop.setEnabled(false);
		
	}
	
	/** Sets the GUI to start state (state when sound is running normally)
	 */
	public void setStartState() {
		
		loadFile.setEnabled(false);
		start.setEnabled(false);
		
		haar.setEnabled(true);
		daubechies.setEnabled(true);
		
		pause.setEnabled(true);
		stop.setEnabled(true);
		
		stop(false);
		
		
	}
	
	/** Sets the GUI to pause state (when sound is paused)
	 */
	public void setPauseState() {
		
		stop.setEnabled(!isPaused);
		pause.setText(isPaused?"UnPause": "Pause");		
	}
	
	/** Sets the GUI to stop state (or normal idle state)
	 */
	public void setStopState() {
		
		loadFile.setEnabled(true);
		subControlPanel.setEnabled(true);
		start.setEnabled(true);
		
		
		pause.setEnabled(false);
		stop.setEnabled(false);
		
	
	}
	
	/** A boolean getter
	 * 
	 * @return boolean value to say whether the sound views should be shown
	 */
	boolean showView()
	{
		return !hideView.isSelected();
	}

	/** A boolean getter
	 * 
	 * @return boolean value to say whether the mic or the file is the source
	 */
	boolean sourceMic()
	{
		return sourceMic.isSelected();
	}
	
	/** A boolean getter
	 * 
	 * @return boolean value to say whether the crossSynthesis box is selected
	 */
	boolean crossSynthesize()
	{
		return crossSynthesize.isSelected();
	}
	
	/** A boolean getter
	 * 
	 * @return boolean value to say whether the denoise box is selected
	 */
	boolean denoise()
	{
		return denoise.isSelected();
	}
	/** A boolean getter
	 * 
	 * @return boolean value to say whether the threshold box is selected
	 */
	boolean threshold()
	{
		return threshold.isSelected();
	}
	/** A boolean getter
	 * 
	 * @return boolean value to say whether the removeClicks box is selected
	 */
	boolean removeClicks()
	{
		return removeClicks.isSelected();
	}
	
	/** A boolean getter
	 * 
	 * @return boolean value to say whether the compression box is selected
	 */
	boolean simulateCompression()
	{
		return simulateCompression.isSelected();
	}
	
	/** A boolean getter
	 * 
	 * @return boolean value to say whether the modulate box is selected
	 */
	boolean modulate()
	{
		
		return modulate.isSelected();
	}
	
	/** A boolean getter
	 * 
	 * @return boolean value to say whether the filter box is selected
	 */
	boolean filter()
	{
		return filter.isSelected();
	}

	/** Returns the numbers in the filter boxes in GUI
	 * 
	 * @return an int[] of 10 filter coefficients
	 */
	int[] returnFilterCoefficients()
	{
		int[] coefficients = new int[filterSpinners.length];
		
		for (int i = 0; i < filterSpinners.length; i++)
		{
			coefficients[filterSpinners.length - i - 1] = ((Integer) filterSpinners[i].getValue()).intValue();
		}
		return coefficients;
	}

	/** Accessor function that indicates whether "haar" button selected
	 * @return boolean isHaar if true, haar search is to be done
	 */
	boolean isHaar() {
		return haar.isSelected();
	}
	

	/** Checks the value of isStopped
	 * @return boolean isStopped true if stopped;
	 */
	boolean isStopped() {
		return isStopped;
	}
	
	/** Checks the value of isPaused
	 * @return boolean isPaused true if paused;
	 */
	boolean isPaused() {
		return isPaused;
	}
	
	/** This method attempts to pause the sound
	 * @param boolean yes true if want to pause
	 */
	void pause(boolean yes) {	
		isPaused = yes;
	}
	
	/** This method attempts to stop the sound
	 * @param boolean yes true if want to stop search
	 */
	void stop(boolean yes) {
		isStopped = yes;
	}
	
	/** Clears the canvas of the previous display
	 */
	void clearCanvas(int size) {
		canvas3.clearCanvas(size);
	}
	
	/** Graphically displays the sound on Canvas 1
	 * @param short[] xCoord array of x-coordinates of the samples
	 * @param short[] yCoord array of y-coordinates of the samples
	 * @param short size number of samples
	 */
	public void displaySignal1(float[] xCoord, float[] yCoord, short size)
	{
		
		canvas.redraw(xCoord, yCoord, size);
		//canvas.update(canvas.getGraphics());	
	}
	
	/** Graphically displays the coefficients on Canvas 2
	 * @param short[] xCoord array of x-coordinates of the samples
	 * @param short[] yCoord array of y-coordinates of the samples
	 * @param short size number of samples
	 */
	public void displaySignal2(float[] xCoord, float[] yCoord, short size)
	{
		
		canvas2.redraw(xCoord, yCoord, size);
		//canvas2.update(canvas2.getGraphics());	
	}
	
	/** Graphically displays the coefficients on Canvas 3
	 * @param short[] coefficients array of wavelet coefficients
	 * @param short size number of samples
	 */
	public void displaySignal3(float[] coefficients, short size)
	{
		
		canvas3.redraw(coefficients, size);
		//canvas3.update(canvas3.getGraphics());	
	}
	
}
