/** This class makes a canvas on which to draw the wavelet coefficient.  It was inspired 
 * and adapted from a Sun Java applet example called 'Arc Test'
 * @author Arthur Asuncion
 */

import java.awt.*;


public class WaveletCanvas extends Canvas {
	
	// size of box, and offsets of the x and y axes
	static final int BOX_SIZE = 20;
	static final int X_AXIS_OFFSET = 160;
	static final int Y_AXIS_OFFSET = 140;
	
	static final int DISPLAY_LIMIT = 1;
	static final int DISPLAY_THRESHOLD = 256;
	
	boolean drawGrid = false;
	boolean drawSignal = false;
	boolean drawLabels = false;
	
	double costOfTour;
	int[][] xCoordBuffer;
	int[][] yCoordBuffer;
	int[][] zCoordBuffer;

	int[] xCoordinates;
	int[] yCoordinates;
	short nSamples;
	
	
	/** Paints the canvas according to its internal information
	 * @param Graphics the graphic on which to paint the picture
	 */
	public void paint(Graphics g){
		
		Rectangle r = getBounds();

		this.setBackground(Color.black);
		
		// draws the grid
		if (drawGrid){
			
			int xLines = r.height / BOX_SIZE;
			int yLines = r.width / BOX_SIZE;
			
			g.setColor(Color.lightGray);
			
			for (int i = 0; i <= xLines; i++) {
				g.drawLine(0, i * BOX_SIZE, r.width, i * BOX_SIZE);
			}
			
			for (int i = 0; i <= yLines; i++) {
				g.drawLine(i * BOX_SIZE, 0, i * BOX_SIZE, r.height);
			}

		}
		
		// draws the signal
		if (drawSignal) {

			float alpha;
			
			for (int n = 0; n < 4; n++) 
			{
				for (int i = 1; i < DISPLAY_THRESHOLD; i ++){
					alpha = (float) (Math.abs((zCoordBuffer[n][i] / (double) Short.MAX_VALUE)));
					if (alpha > 1) alpha =1;
					
					g.setColor(Color.getHSBColor((float)0.5,(float)0.5, alpha));
					g.fillRect((int) (xCoordBuffer[n][i] + n * 5.5 * BOX_SIZE), yCoordBuffer[n][i] - 1, 10, 10);
					
				}
				
				// Not drawing everything (drawing only every 16th sample)! 
				// If I drew everything, it would take too much memory and time.
				
				for (int i = DISPLAY_THRESHOLD; i < nSamples/DISPLAY_LIMIT; i = i + 16) {
					
					alpha = (float) (Math.abs((zCoordBuffer[n][i] / (double) Short.MAX_VALUE)));
					if (alpha > 1) alpha =1;
					
					g.setColor(Color.getHSBColor((float)0.5,(float)0.5, alpha));
					g.fillRect((int) (xCoordBuffer[n][i] + n * 5.5 * BOX_SIZE), yCoordBuffer[n][i] - 1, 10, 10);
				}
			}
		}
		
		g.setColor(Color.getHSBColor((float)0.5,(float)0.5, 1));
		g.drawString("Triangular Visualization: ", X_AXIS_OFFSET - 7 * BOX_SIZE, BOX_SIZE);
	}
	
	
	/** Redraws the canvas with the specified coefficient parameters
	 * This uses a very complicated algorithm that I spent a lot of time on to make the 
	 * triangular visualization
	 * 
	 * @param coefficients wavelet coefficients
	 * @param size the number of coefficients
	 */
	public void redraw(float[] coefficients, short size) 
	{
		drawSignal = true;
		nSamples = size;

		int levels = (int) Math.round( (Math.log(size) / Math.log(2)));
		
		float[] xCoord = new float[size];
		float[] yCoord = new float[size];
		int[] zCoord = new int[size];
		

		// Complicated algorithm to calculate the x-Coordinate of a particular coefficient
		
		for (int i = 1; i < DISPLAY_THRESHOLD; i ++)
		{
			int numerator = (i % (int) Math.pow(2, (int) (Math.floor(Math.log(i)/Math.log(2)))) + 1);
			int denominator = (int) Math.pow(2, Math.round((Math.floor( (int) (Math.log(i)/ Math.log(2))))) + 1);
		
			
			xCoord[i] = (float) ((numerator * 2 - 1) *  (50. / denominator));
			
			yCoord[i] = (float) (5 * (levels - Math.floor( (int) (Math.log(i)/Math.log(2)))));
			
			zCoord[i] = (int) coefficients[i];
		}
		
		for (int i = DISPLAY_THRESHOLD; i < size / DISPLAY_LIMIT; i = i + 16)
		{
			int numerator = (i % (int) Math.pow(2, (int) (Math.floor(Math.log(i)/Math.log(2)))) + 1);
			int denominator = (int) Math.pow(2, Math.round((Math.floor( (int) (Math.log(i)/ Math.log(2))))) + 1);
		
			
			xCoord[i] = (float) ((numerator * 2 - 1) *  (50. / denominator));
			
			yCoord[i] = (float) (5 * (levels - Math.floor( (int) (Math.log(i)/Math.log(2)))));
			
			zCoord[i] = (int) coefficients[i];
		}
				
		
		// have to transform the coordinates
		Doubleton transformedCoordinates = 
				transformCoordinates(xCoord, yCoord, size);
		xCoordinates = (int[])transformedCoordinates.getFirst();
		yCoordinates = (int[])transformedCoordinates.getSecond();
		
		
		// Here I am moving the coordinate arrays around so that the visualization
		// would show the coefficients moving from right to left
		for (int i = 0; i < 3; i++)
		{
			xCoordBuffer[i] = xCoordBuffer[i+1];
			yCoordBuffer[i] = yCoordBuffer[i+1];
			zCoordBuffer[i] = zCoordBuffer[i+1];
		}
		
		xCoordBuffer[3]= xCoordinates;
		yCoordBuffer[3]= yCoordinates;
		zCoordBuffer[3]= zCoord;
		
		repaint();
	}
	
	
	/** Draws/Undraws the grid
	 * @param boolean grid true if grid is to be drawn
	 */
	public void drawGrid(boolean grid) {	
		drawGrid = grid;
		repaint();
	}
	
	
	/** Draws/Undraws the labels
	 * @param boolean label true if labels are to be drawn 
	 */
	public void drawLabels(boolean label) {
		drawLabels = label;
		repaint();
	}
	
	
	/** Clears the canvas of the previous tour
	 * 
	 * @param size
	 */
	public void clearCanvas(int size) {	
		
		xCoordBuffer = new int[4][size];
		yCoordBuffer = new int[4][size];
		zCoordBuffer = new int[4][size];
		
		for (int i = 0; i < 4; i++)
		{
			xCoordBuffer[i] = new int[size];
			yCoordBuffer[i] = new int[size];
			zCoordBuffer[i] = new int[size];
		}
		
		drawSignal = false;
		costOfTour = 0;
		repaint();
	}
	
	/** Transforms the coordinates to make it look right on the GUI
	 * @param float[] xCoord x-coordinates to transform
	 * @param float[] yCoord y-coordinates to transform
	 * @param short numberNodes number of points
	 * @return Doubleton the transformed x and y coordinates
	 */
	Doubleton transformCoordinates(float[] xCoord, float[] yCoord, 
									short numberNodes){
		
		int[] transXCoord = new int[numberNodes];
		int[] transYCoord = new int[numberNodes];
		
		// Logic here
		for (int i = 0; i < numberNodes; i++) {
			transXCoord[i] = (int)Math.floor(BOX_SIZE * xCoord[i] / 10.0) 
							+ X_AXIS_OFFSET;
			transYCoord[i] = -1 * (int)Math.floor(BOX_SIZE * yCoord[i] / 10.0) 
							+ Y_AXIS_OFFSET;
		}
		
		return new Doubleton(transXCoord, transYCoord);
	}
	
}
