/**
 * @author Arthur Asuncion
 *
 * This class contains some effects processing functions for ordered wavelet coefficients
 * like thresholding.
 */
public class WaveletEffects {
	
	/** This field is used to maintain a count of samples processed */
	int count;
	
	/** This field holds the modulation parameter.  1 is the slowest parameter
	 * while 5 is the highest.
	 */
	static final int MODULATION_COEFFICIENT=3;
	
	/** This field holds a threshold parameter for the threshold function
	 * 
	 */
	static final int THRESHOLD=1000;
	
	/** This is the constructor
	 * 
	 * @param count
	 */
	public WaveletEffects(int count){
		this.count = count;
	}
	
	/** This method crossSynthesizes two coefficient sets by thresholding 
	 * @param s one coefficient set
	 * @param sCS another coefficient set
	 * @param size size of sets
	 */
	void crossSynthesize(float[] s, float[] sCS, int size)
	{

		for (int i = 0; i < size; i++)
		{

			if (Math.abs(sCS[i]) < 1000) {
				//orig[i] = (normalize(orig[i]) * normalize(cross[i]));
				s[i] = (float) (0.05 * s[i]);
			}
			
			else if (Math.abs(sCS[i]) < 3000) {

				s[i] = (float) (0.2 * s[i]);
			}
			
			else if (Math.abs(sCS[i]) < 6000) {

				s[i] = (float) (0.4 * s[i]);

			}
			else {

				s[i] = (float) (0.7 * s[i]);

			}

		}
		
	}
	

	/** This method modulates the signal by multiplying it to a cosine wave
	 * 
	 * @param s the coefficient array
	 * @param size the size of the array
	 */
	void modulate(float[] s, int size)
	{	
		int logSize = 0;
		double log2 = Math.log(2);
		for (int i = 0; i < size/4; i++)
		{	
			s[i] = (float) (0.3 * s[i] + 0.4 * s[i] * Math.sin(count / (MODULATION_COEFFICIENT * 50.)));
			
			if (Math.log(size) / log2 > logSize + 1) {
				count++;
				logSize++;
			}
		}
		
		
	}
	
	/** This method filters the frequency channels according to the parameters given
	 * by the user in the AWE GUI 
	 * 
	 * @param s the float[] of coefficients
	 * @param size array size
	 * @param parameters given by user in the GUI
	 */
	void filter(float[] s, int size, int[] parameters)
	{
		// Since the frequency channels are logarithmically spaced, some care needs
		// to be taken to map the right parameter to the right channel
		for (int i = 0; i < parameters.length - 1; i++)
		{
			
			for (int j = size/2; j < size; j++) {
			
				s[j] = (float) (s[j] * parameters[i] / 100.);
			}
			
			size = size / 2;
		}
		
		
		for (int i = 0; i < size; i++)
		{
			s[i] = (float) (s[i] * parameters[parameters.length - 1] / 100.);
		}
		
	}
	
	/** This thresholds the coefficients
	 * 
	 * @param s the float array
	 * @param size array size
	 */
	void threshold(float[] s, int size)
	{
		for (int i = 1; i < s.length; i++)
		{
			if (Math.abs(s[i]) < THRESHOLD) s[i] = 0;
			else s[i] = s[i] - THRESHOLD;
		}
	}
	
	/** This method seeks to simulate compression (by a factor of 16)
	 * 
	 * @param s float of array of the coefficients
	 * @param size size of array
	 */
	void simulateCompression(float[] s, int size)
	{
		for (int i = size / 8; i < s.length; i++)
		{
			s[i] = 0;
		}
		
		for (int i = 0; i < size / 8; i++)
		{
			s[i] = (int) (s[i] / (double) (Math.pow(2, 8)));
			s[i] = (float) (s[i] * Math.pow(2, 8));
		}
	}
	
	
	/** This is a soft threshold function for unoredered (butterfly-pattern) coefficients
	 * (not used in AWE interface)
	 * @param s float array of unordered coefficients
	 * @param size size of array
	 */ 
	void thresholdUnordered(float[] s, int size)
	{
		
		float ave = s[0];
		ave = ave/10;
		
		for (int i = 1; i < s.length; i = i + 1)
		{
			if (s[i] < ave) s[i] =  0;
			else s[i] = s[i] - ave;
			
			count++;
		}
	}
	
	/** This method denoises the array by removing the highest frequency (the small changes
	 * in the signal)
	 * 
	 * @param s array to denoise
	 * @param size array's size
	 */
	void denoise(float[] s, int size)
	{
		for (int i = size/2; i < size; i++){
			s[i] = 0;
		}
	}
	
	
	/** This method removes edge clicks via a short linear ramping window
	 * 
	 * @param f the float array
	 * @param size array's size
	 */
	void removeEdgeClicks(float[] f, int size)
	{
		
		// removes edge clicks
		for (int i = 0; i < 20; i++) {
			f[i] = (float) (f[i] * (i*i / (20.*20.)));
			//f[i] = (float) (f[i] * i / 100.);
		}
		for (int i = f.length - 21; i < f.length; i++) {
			f[i] = (float) (f[i] * Math.pow((f.length - i - 1),2) / (20.*20.));
			//f[i] = (float) (f[i] * (f.length - i - 1) / 100.);
		}
		
	}
	
	static float normalize (float f)
	{
		return (float) (f / (double) Short.MAX_VALUE);
	}
	
	static float denormalize (float f)
	{
		return f * Short.MAX_VALUE;
	}
	

}
